package org.nci.dts.client;

import com.apelon.dts.client.DTSException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.apache.log4j.Category;
import java.sql.Date;
/**
 * <p>A CodeActionDate contains a concept code and a baseline date.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version DTS 3.0
 */

public class CodeActionDate
{
	private static Category cat = Category.getInstance(CodeActionDate.class.getName());
        private String code;
        private Date date;

	/**
         * Contructs a CodeActionDate which has a concept code and a baseline date.
         *
         * @param         code        concept code of the CodeActionDate
         * @param         date        baseline date of the CodeActionDate
         *
         * @since         DTS 2.4
         */
         protected CodeActionDate(String code, Date date)
	{
		this.code = code;
		this.date = date;
	}

        /**
         * Gets the concept code for this CodeActionDate.
         *
         * @return        concept code for this CodeActionDate.
         *
         * @since         DTS 2.4
         */
	public String getCode()
	{
		return code;
	}

        /**
         * Gets the baseline date for this CodeActionDate.
         *
         * @return        baseline date for this CodeActionDate.
         *
         * @since         DTS 2.4
         */
	public Date getDate()
	{
		return date;
	}

        /**
        * Creates a CodeActionDate by passing an XML Document.
        *
        * @param       doc       the XML document of the CodeActionDate.
        * @return      the created CodeActionDate.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static CodeActionDate createCodeActionDate(Document doc) throws DTSException
        {
                Element elem = doc.getDocumentElement();
                cat.debug("CODEACTIONDATE: " + elem.getNodeValue());
                return createCodeActionDate(elem);
        }

        /**
        * Creates a CodeActionDate by passing an XML Element. The Element contains
        * the information(s) about the CodeActionDate.
        *
        * @param       elem       the XML Element of the CodeActionDate.
        * @return      the created CodeActionDate.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static CodeActionDate createCodeActionDate(Element elem) throws DTSException
        {
                String tag_name = elem.getTagName().toString().toUpperCase();
                if (tag_name.equals("NULL"))
                        return null;
                if (tag_name.equals("CODEACTIONDATE"))
                {
                        String s1 = elem.getChildNodes().item(0).getChildNodes().item(0).getNodeValue();
                        String s2 = elem.getChildNodes().item(1).getChildNodes().item(0).getNodeValue();
                        Date date = java.sql.Date.valueOf(s2);
                        CodeActionDate codeactiondate = new CodeActionDate(s1, date);
                        return codeactiondate;
                }
                else
                {
                        throw new DTSException("Expected a CODEACTIONDATE tag");
                }
        }

        /**
        * Creates CodeActionDates by passing an XML Document.
        *
        * @param       doc       the XML document of CodeActionDates.
        * @return      an array of the created CodeActionDates.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static CodeActionDate[] createCodeActionDates(Document doc) throws DTSException
        {
                Element elem = doc.getDocumentElement();
                cat.debug("CODEACTIONDATES: " + elem.getNodeValue());
                return createCodeActionDates(elem);
        }

        /**
        * Creates CodeActionDates by passing an XML Element. The Element contains
        * informations about CodeActionDates.
        *
        * @param       elem       the XML Element of CodeActionDates.
        * @return      an array of the created CodeActionDates.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static CodeActionDate[] createCodeActionDates(Element elem) throws DTSException
        {
                String tag_name = elem.getTagName().toString().toUpperCase();
                if(tag_name.equals("NULL")) {
                        return new CodeActionDate[0];
                }
	        else if (tag_name.equals("CODEACTIONDATES"))
                {
                        int n = elem.getChildNodes().getLength();
                        CodeActionDate[] codeactiondates = new CodeActionDate[n];
                        int i;
                        for (i=0; i<n; i++) {
	                        codeactiondates[i] = createCodeActionDate((Element)elem.getChildNodes().item(i));
                        }
                        return codeactiondates;
                }
                else
                {
                        throw new DTSException("Expected a CODEACTIONDATES tag");
                }
        }
}
