package org.nci.dts.client;

import com.apelon.apelonserver.client.*;
import com.apelon.dts.client.DTSException;
import com.apelon.common.xml.XML;
import com.apelon.common.util.ApelUtil;
import org.nci.dts.EditActionConstants;

import java.sql.Date;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
/**
 * <p>A ConceptHistoryServer provides common methods for getting concept history.
 * This class is the main class in org.nci.dts.client package. </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version DTS 3.0
 */

public abstract class ConceptHistoryServer extends BasicQuery
{
        /**
        * Creates an instance of ConceptHistoryServer.
        * <br>Depending on the type of ServerConnection object passed in the
        * argument, the server may be local (JDBC), socket, or another implementation.
        *
        * @param         sc              server connectin type
        * @return        the created ConceptHistoryServer
        * @exception     DTSException    error
        *
        * @see           com.apelon.apelonserver.client.ServerConnectionJDBC
        * @see           com.apelon.apelonserver.client.ServerConnectionSocket
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public static ConceptHistoryServer createInstance(ServerConnection sc)
            throws DTSException
        {
                ConceptHistoryServer new_srvr = null;
                if( sc == null ) {
                        throw new DTSException("ConceptHistoryServer.createInstance() called with null argument");
                }
                new_srvr = new ConceptHistoryServerImpl(sc);
                new_srvr.setVersion("DTS:021");
                return new_srvr;
        }
        /**
        * Creates an instance of ConceptHistoryServer.
        * <br>Depending on the type of ServerConnection object passed in the
        * argument, the server may be local (JDBC), socket, or another implementation.
        * <br>second argument is a boolean value for xml validation.
        *
        * @param         sc              server connectin type
        * @param         isValidating    a boolean value. It is true if xml needs to
        *                                be validated, otherwise it is false.
        * @return        the created ConceptHistoryServer
        * @exception     DTSException    error
        *
        * @see           com.apelon.apelonserver.client.ServerConnectionJDBC
        * @see           com.apelon.apelonserver.client.ServerConnectionSocket
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public static ConceptHistoryServer createInstance(ServerConnection sc, boolean isValidating)
            throws DTSException
        {
                ConceptHistoryServer new_srvr = null;
                if( sc == null ) {
                        throw new DTSException("ConceptHistoryServer.createInstance() called with null argument");
                }
                new_srvr = new ConceptHistoryServerImpl(sc, isValidating);
                new_srvr.setVersion("DTS:021");
                return new_srvr;
        }

        /**
        * Gets all distinct dates in concept_history table.
        *
        * @return        an array of Date   All distinct dates in concept_history table.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public Date[] getHistoryDates()
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getDate", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getDate>";
                query += "<date value=\"all\"/>";
                query += "</getDate>";
                try {
                        return createDates(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets the initial date in concept_history table.
        *
        * @return        Date               The initial date in the concept_history table.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public Date getHistoryStart()
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getDate", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getDate>";
                query += "<date value=\"start\"/>";
                query += "</getDate>";
                try {
                        return createDate(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets the latest date in concept_history table.
        *
        * @return        Date               The latest date in the concept_history table.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public Date getHistoryEnd()
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getDate", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getDate>";
                query += "<date value=\"end\"/>";
                query += "</getDate>";
                try {
                        return createDate(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets the baseline date of creation of the specified concept.
        *
        * @param         namespaceId        The namespace identifier of the concept
        * @param         inCode             The specified concept's code.
        * @return        Date               The baseline date of creation of inCode
        *                                   from either "Create" or "Split" edit actions.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public Date getConceptCreationDate(int namespaceId, String inCode)
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getConceptDate", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getConceptDate>";
                query += "<createDate/>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
                query += inCode;
                query += "</conceptCode>";
                query += "</getConceptDate>";
                try {
                        return createDate(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets the descendant concept codes of the specified concept. The search extends
        * to a final baseline date starting from an initial baseline date. The search is
        * based on a boolean value. If the value is true, the method only searches for
        * the active concepts at the final baseline date. If the value is false, the method
        * searches for all descendant concepts, whether active or retired.
        * The search is only applied to "split/merge" actions and the specified concept
        * could be the descendant concept of itself.
        *
        * @param         namespaceId        The namespace identifier of the concept
        * @param         inCode             The specified concept's code.
        * @param         atBaseline         The boolean value to specify the search type.
        * @param         iBaselineDate      The initial baseline date.
        * @param         fBaselineDate      The final baseline date.
        * @return        an array of String The returned concept codes which are the descendant
        *                                   concepts of inCode.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public String[] getDescendantCodes(int namespaceId, String inCode, boolean atBaseline, Date iBaselineDate, Date fBaselineDate)
            throws DTSException
        {
                String boolean_str = null;
                if (atBaseline) {
                        boolean_str = "true";
                } else {
                        boolean_str = "false";
                }
                String query = "";
                query += XML.createProlog("getCodesInDateRange", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getCodesInDateRange>";
                query += "<descendant/>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
                query += inCode;
                query += "</conceptCode>";
                query += "<boolean>";
                query += boolean_str;
                query += "</boolean>";
                query += "<startDate>";
                query += iBaselineDate.toString();
                query += "</startDate>";
                query += "<endDate>";
                query += fBaselineDate.toString();
                query += "</endDate>";
                query += "</getCodesInDateRange>";
                try {
                        return createConceptCodes(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets the descendant concepts' codes of the specified concept involved in
        * "split or merge" actions that occurred in the specified baseline date.
        * The specified concept is included if it's not retired in the specified
        * baseline date.
        *
        * @param         namespaceId        The namespace identifier of the concept
        * @param         inCode             The specified concept's code.
        * @param         baselineDate       The specified baseline date.
        * @return        an array of String The returned concept codes are the descendant
        *                                   concepts of inCode which are involved in
        *                                   "split or merge" actions that occurred in baselineDate.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public String[] getCodeActionChildren(int namespaceId, String inCode, Date baselineDate)
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getCodesInDate", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getCodesInDate>";
                query += "<descendant/>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
                query += inCode;
                query += "</conceptCode>";
                query += "<baselineDate>";
                query += baselineDate.toString();
                query += "</baselineDate>";
                query += "</getCodesInDate>";
                try {
                        return createConceptCodes(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets the descendant concepts' CodeActionDate of the specified concept. A CodeActionDate
        * contains a concept code and a baseline date. The method searches for all descendant
        * concepts of the specified concept, whether active or retired.
        * The search is only applied to the "split/merge" actions and the specified concept
        * could be the descendant concept of itself.
        *
        * @param         namespaceId        The namespace identifier of the concept
        * @param         inCode             The specified concept's code.
        * @param         iBaselineDate      The initial baseline date.
        * @param         fBaselineDate      The final baseline date.
        * @return        an array of CodeActionDate     The returned CodeActionDates contain the
        *                                               descendant concepts of inCode and the
        *                                               correspond baseline dates.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public CodeActionDate[] getDescendantCodeActionDates(int namespaceId, String inCode, Date iBaselineDate, Date fBaselineDate)
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getCodeActionDate", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getCodeActionDate>";
                query += "<descendant/>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
                query += inCode;
                query += "</conceptCode>";
                query += "<startDate>";
                query += iBaselineDate.toString();
                query += "</startDate>";
                query += "<endDate>";
                query += fBaselineDate.toString();
                query += "</endDate>";
                query += "</getCodeActionDate>";
                try {
                        return CodeActionDate.createCodeActionDates(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets the ancestor concept codes of the specified concept. The search extends
        * to a final baseline date starting from an initial baseline date. The search is
        * based on a boolean value. If the value is true, the method only searches for
        * the active concepts at the initial baseline date. If the value is false, the method
        * searches for all ancestor concepts, whether active or retired.
        * The search is only applied to "split/merge" actions and the specified concept
        * could be the ancestor concept of itself.
        *
        * @param         namespaceId        The namespace identifier of the concept
        * @param         inCode             The specified concept's code.
        * @param         atBaseline         The boolean value to specify the search type.
        * @param         iBaselineDate      The initial baseline date.
        * @param         fBaselineDate      The final baseline date.
        * @return        an array of String The returned concept codes which are the ancestor
        *                                   concepts of inCode.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public String[] getAncestorCodes(int namespaceId, String inCode, boolean atBaseline, Date iBaselineDate, Date fBaselineDate)
            throws DTSException
        {
                String boolean_str = null;
                if (atBaseline) {
                        boolean_str = "true";
                } else {
                        boolean_str = "false";
                }
                String query = "";
                query += XML.createProlog("getCodesInDateRange", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getCodesInDateRange>";
                query += "<ancestor/>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
                query += inCode;
                query += "</conceptCode>";
                query += "<boolean>";
                query += boolean_str;
                query += "</boolean>";
                query += "<startDate>";
                query += iBaselineDate.toString();
                query += "</startDate>";
                query += "<endDate>";
                query += fBaselineDate.toString();
                query += "</endDate>";
                query += "</getCodesInDateRange>";
                try {
                        return createConceptCodes(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets the ancestor concepts' codes of the specified concept involved in
        * "split or merge" actions that occurred in the specified baseline date.
        * The specified concept is included if it's an ancestor of itself.
        *
        * @param         namespaceId        The namespace identifier of the concept
        * @param         inCode             The specified concept's code.
        * @param         baselineDate       The specified baseline date.
        * @return        an array of String The returned concept codes are the ancestor
        *                                   concepts of inCode which are involved in
        *                                   "split or merge" actions that occurred in baselineDate.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public String[] getCodeActionParents(int namespaceId, String inCode, Date baselineDate)
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getCodesInDate", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getCodesInDate>";
                query += "<ancestor/>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
                query += inCode;
                query += "</conceptCode>";
                query += "<baselineDate>";
                query += baselineDate.toString();
                query += "</baselineDate>";
                query += "</getCodesInDate>";
                try {
                        return createConceptCodes(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets the ancestor concepts' CodeActionDate of the specified concept. A CodeActionDate
        * contains a concept code and a baseline date. The method searches for all ancestor
        * concepts of the specified concept, whether active or retired.
        * The search is only applied to "split/merge" actions and the specified concept
        * could be the ancestor concept of itself.
        *
        * @param         namespaceId        The namespace of the concept
        * @param         inCode             The specified concept's code.
        * @param         iBaselineDate      The initial baseline date.
        * @param         fBaselineDate      The final baseline date.
        * @return        an array of CodeActionDate     The returned CodeActionDates contain the
        *                                               ancestor concepts of inCode and the
        *                                               correspond baseline dates.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public CodeActionDate[] getAncestorCodeActionDates(int namespaceId, String inCode, Date iBaselineDate, Date fBaselineDate)
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getCodeActionDate", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getCodeActionDate>";
                query += "<ancestor/>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
                query += inCode;
                query += "</conceptCode>";
                query += "<startDate>";
                query += iBaselineDate.toString();
                query += "</startDate>";
                query += "<endDate>";
                query += fBaselineDate.toString();
                query += "</endDate>";
                query += "</getCodeActionDate>";
                try {
                        return CodeActionDate.createCodeActionDates(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets all dates in which a specified action takes place on the specified concept.
        * The returned dates might not be distinct when the specified action is "split/merge".
        *
        * @param         namespaceId       The namespace of the concept
        * @param         inCode             The specified concept's code.
        * @param         iAction            The specified editing action.
        * @return        an array of Date   All dates in which iAction takes place on inCode.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public Date[] getConceptEditActionDates(int namespaceId, String inCode, int iAction)
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getConceptDate", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getConceptDate>";
                query += "<editDate value=\"action\">";
                query += EditActionConstants.actionToString(iAction);
                query += "</editDate>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
                query += inCode;
                query += "</conceptCode>";
                query += "</getConceptDate>";
                try {
                        return createDates(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets all EditAction of the specified concept. A EditAction contains a edit date and
        * an editing action. The method searches for all editing actions and the corresponding
        * editing dates on the specified concept.
        *
        * @param         namespaceId       The namespace of the concept
        * @param         inCode             The specified concept's code.
        * @return        an array of EditAction     The returned EditActions contain all editing
        *                                           actions and the corresponding editing dates
        *                                           on inCode.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public EditAction[] getConceptEditAction(int namespaceId, String inCode)
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getEditAction", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getEditAction>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
                query += inCode;
                query += "</conceptCode>";
                query += "</getEditAction>";
                try {
                        return EditAction.createEditActions(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Checks if the specified concept is retired or not.
        *
        * @param         namespaceId       The namespace of the concept
        * @param         inCode             The specified concept's code.
        * @return        a boolean          True - inCode is retired.
        *                                   False - inCode is active.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public boolean isConceptRetired(int namespaceId, String inCode)
            throws DTSException
        {
                String query = "";
		query += XML.createProlog("isRetired", org.nci.dts.dtd.DTD.QUERY, false);
		query += "<isRetired>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
		query += inCode;
		query += "</conceptCode>";
		query += "</isRetired>";
                try {
		        return createBoolean(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Checks if the specified concept is edited or not during a date range. The check is
        * depended on a boolean value. If the boolean value is true, the method only checks
        * if the specified concept was merged or split in the date range. If the boolean
        * value is false, the method checks if the specified concept was edited in any way
        * in the date range.
        *
        * @param         namespaceId       The namespace of the concept
        * @param         inCode             The specified concept's code.
        * @param         mrg_splt           True - only check for "merge or split" editing action.
        *                                   False - check for any editing action.
        * @param         iBaselineDate      The initial baseline date.
        * @param         fBaselineDate      The final baseline date.
        * @return        a boolean          True - inCode is edited.
        *                                   False - inCode isn't edited.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public boolean isConceptEdited(int namespaceId, String inCode, boolean mrg_splt, Date iBaselineDate, Date fBaselineDate)
            throws DTSException
        {
                String boolean_str = null;
                if (mrg_splt) {
                        boolean_str = "true";
                } else {
                        boolean_str = "false";
                }
                String query = "";
		query += XML.createProlog("isEdited", org.nci.dts.dtd.DTD.QUERY, false);
		query += "<isEdited>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
		query += inCode;
		query += "</conceptCode>";
                query += "<boolean>";
                query += boolean_str;
                query += "</boolean>";
                query += "<startDate>";
                query += iBaselineDate.toString();
                query += "</startDate>";
                query += "<endDate>";
                query += fBaselineDate.toString();
                query += "</endDate>";
		query += "</isEdited>";
                try {
		        return createBoolean(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets the number of editing actions on the specified concept registered in the
        * specified baseline date.
        *
        * @param         namespaceId       The namespace of the concept
        * @param         inCode             The specified concept's code.
        * @param         baselineDate       The specified baseline date.
        * @return        an int             The number of editing actions on inCode
        *                                   registered in baselineDate.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public int getNumberEditActions(int namespaceId, String inCode, Date baselineDate)
            throws DTSException
        {
                String query = "";
		query += XML.createProlog("getAction", org.nci.dts.dtd.DTD.QUERY, false);
		query += "<getAction type=\"count\">";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
		query += inCode;
		query += "</conceptCode>";
                query += "<baselineDate>";
                query += baselineDate.toString();
                query += "</baselineDate>";
		query += "</getAction>";
                try {
		        return Integer.parseInt(createString(executeQuery(query)));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets the number of editing actions on the specified concept registered in the
        * specified date range.
        *
        * @param         namespaceId        The namespace identifier of the concept
        * @param         inCode             The specified concept's code.
        * @param         iBaselineDate      The initial baseline date.
        * @param         fBaselineDate      The final baseline date.
        * @return        an int             The number of editing actions on inCode
        *                                   registered from iBaselineDate to fBaseline.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public int getNumberEditActions(int namespaceId, String inCode, Date iBaselineDate, Date fBaselineDate)
            throws DTSException
        {
                String query = "";
		query += XML.createProlog("getActionInDateRange", org.nci.dts.dtd.DTD.QUERY, false);
		query += "<getActionInDateRange>";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
		query += inCode;
		query += "</conceptCode>";
                query += "<startDate>";
                query += iBaselineDate.toString();
                query += "</startDate>";
                query += "<endDate>";
                query += fBaselineDate.toString();
                query += "</endDate>";
		query += "</getActionInDateRange>";
                try {
		        return Integer.parseInt(createString(executeQuery(query)));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Gets all edit actions of the specified concept registered in the specified
        * baseline date. The result is an array of Integer which is corresponding to
        * the editing action constants defined in EditActionConstants class.
        *
        * @param         namespaceId        The namespace identifier of the concept
        * @param         inCode             The specified concept's code.
        * @param         baselineDate       The specified baseline date.
        * @return        an array of int    All edit action constants of inCode registered
        *                                   in baselineDate.
        * @exception     DTSException       error
        *
        * @since         DTS 2.4
        * @version       DTS 3.0
        */
        public int[] getConceptEditAction(int namespaceId, String inCode, Date baselineDate)
            throws DTSException
        {
                String query = "";
		query += XML.createProlog("getAction", org.nci.dts.dtd.DTD.QUERY, false);
		query += "<getAction type=\"detail\">";
                query += "<conceptCode namespaceid=\"" + namespaceId + "\" >";
		query += inCode;
		query += "</conceptCode>";
                query += "<baselineDate>";
                query += baselineDate.toString();
                query += "</baselineDate>";
		query += "</getAction>";
                try {
		        return createActionConstants(executeQuery(query));
                } catch (ApelonException e) {
                        throw new DTSException("Error executing XML query.", e);
                }
        }

        /**
        * Creates a Date by passing an XML Document.
        * @param       doc    the XML document for the Date.
        * @return      the created Date.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        * @version     DTS 3.0
        */
        private Date createDate(Document doc)
        {
                String date_str = createString(doc);
                Date date = null;
                if (date_str != null) {
                        date = java.sql.Date.valueOf(date_str);
                }
                return date;
        }

        /**
        * Creates an array of Date by passing an XML Document.
        *
        * @param       doc    the XML document for the array of Date.
        * @return      an array of the created Date.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        * @version     DTS 3.0
        */
        private Date[] createDates(Document doc)
        {
                String[] dates_str = createStrings(doc);
                if (dates_str == null) {
                        return null;
                }
                Date[] dates = new Date[dates_str.length];
                for (int i=0; i<dates_str.length; i++) {
                        dates[i] = java.sql.Date.valueOf(dates_str[i]);
                }
                return dates;
        }

        /**
        * Creates a concept code string by passing an XML Document.
        * @param       doc    the XML document for the concept code.
        * @return      the created concept code string.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        * @version     DTS 3.0
        */
        private String createConceptCode(Document doc) {
                return createString(doc);
        }

        /**
        * Creates an array of concept codes by passing an XML Document.
        *
        * @param       doc the XML document for the concept codes.
        * @return      an array of the created concept codes.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        * @version     DTS 3.0
        */
        private String[] createConceptCodes(Document doc) {
                return createStrings(doc);
        }

        /**
        * Creates an edit action constant by passing an XML Document.
        * @param       doc    the XML document for the edit action.
        * @return      the created edit action constant.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        * @version     DTS 3.0
        */
        private int createActionConstant(Document doc) {
                String action_str = createString(doc);
                int action = EditActionConstants.actionToConstant(action_str);
                return action;
        }

        /**
        * Creates an array of edit action constants by passing an XML Document.
        *
        * @param       doc the XML document for the edit actions.
        * @return      an array of the created edit action constants.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        * @version     DTS 3.0
        */
        private int[] createActionConstants(Document doc) {
                String[] actions_str = createStrings(doc);
                if (actions_str == null) {
                        return null;
                }
                int[] actions = new int[actions_str.length];
                for(int i=0; i<actions_str.length; i++) {
                        actions[i] = EditActionConstants.actionToConstant(actions_str[i]);
                }
                return actions;
        }

        /**
        * Creates a string by passing an XML Document.
        * This method is the base method for the createDate, the createConceptCode,
        * and the createActionConstant methods.
        *
        * @param       doc the XML document for the string.
        * @return      the created string.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        * @version     DTS 3.0
        */
        private String createString(Document doc)
        {
                /**
                *  We are expecting following XML :
                *  <string> string </string>
                */
                Element elem = doc.getDocumentElement();
                String tag_name = elem.getTagName().toString().toUpperCase();
                if (tag_name.equals("NULL")) {
                        return null;
                }

                String str = null;
                if( elem.getFirstChild() != null )
                {
                        str = ( elem.getFirstChild() ).getNodeValue();
                }
                return str;
        }

        /**
        * Creates an array of String by passing an XML Document.
        * This method is the base method for the createDates, the createConceptCodes,
        * and the createActionConstants methods.
        *
        * @param       doc the XML document for the strings.
        * @return      the created array of String.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        * @version     DTS 3.0
        */
        private String[] createStrings(Document doc)
        {
                /**
                *  We are expecting following XML tag:
                *  <strings>
                *      <string> string 1 </string>
                *      <string> string 2 </string>
                *  </strings>
                */
                Element elem = doc.getDocumentElement();
                String tag_name = elem.getTagName().toString().toUpperCase();
                if (tag_name.equals("NULL")) {
                        return null;
                }

                NodeList rootChildNodes = elem.getChildNodes();
                String[] strs = new String[ rootChildNodes.getLength() ];
                if( rootChildNodes != null )
                {
                        for( int i=0; i< rootChildNodes.getLength(); i++ )
                        {
                                Node stringNode = rootChildNodes.item( i );
                                if( stringNode != null )
                                {
                                        String str = "";
                                        if( stringNode.getFirstChild() != null )
                                        {
                                                str = stringNode.getFirstChild().getNodeValue();
                                        }
                                        strs[i] = str;
                                }
                                else
                                        strs[i] = "";

                        }
                }
                return strs;
        }

        /**
        * Creates a boolean by passing an XML Document.
        *
        * @param       doc the XML document for the boolean.
        * @return      the created boolean.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        * @version     DTS 3.0
        */
        boolean createBoolean(Document doc) throws DTSException
        {
                Element elem = doc.getDocumentElement();
                //cat.debug("BOOLEAN: " + elem.getNodeValue());
                return createBoolean(elem);
        }

        /**
        * Creates a boolean by passing an XML Element.
        *
        * @param       elem the XML element for the boolean.
        * @return      the created boolean.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        * @version     DTS 3.0
        */
        boolean createBoolean(Element elem) throws DTSException
        {
                String tag_name = elem.getTagName().toString().toUpperCase();
                if (tag_name.equals("BOOLEAN"))
                {
                        String bool_tag = elem.getFirstChild().getNodeName().toString().toUpperCase();
                        return Boolean.valueOf(bool_tag).booleanValue();
                }
                else
                {
                        throw new DTSException("Expected a BOOLEAN tag");
                }
        }
}