package org.nci.dts.client;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.apache.log4j.*;
import java.sql.Date;
import com.apelon.dts.client.DTSException;
/**
 * <p>A DbInfo contains database name, version, and release date.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version DTS 3.0
 */

public class DbInfo {
        private static Category cat = Category.getInstance(DbInfo.class.getName());
        private String name;
        private String version;
        private Date release_date;

        /**
        * Contructs a DbInfo object which has a name, a version, and a date.
        *
        * @param         name            name of the database
        * @param         version         version of the database
        * @param         release_date    release date of the database
        *
        * @since         DTS 2.4
        */
        protected DbInfo(String name, String version, Date release_date)
        {
                this.name = name;
                this.version = version;
                this.release_date = release_date;
        }

        /**
        * Gets the name for this DbInfo.
        *
        * @return        name for this DbInfo.
        *
        * @since         DTS 2.4
        */
        public String getName()
        {
                return name;
        }

        /**
        * Gets the version for this DbInfo.
        *
        * @return        version for this DbInfo.
        *
        * @since         DTS 2.4
        */
        public String getVersion()
        {
                return version;
        }

        /**
        * Gets the release date for this DbInfo.
        *
        * @return        release date for this DbInfo.
        *
        * @since         DTS 2.4
        */
        public Date getReleaseDate()
        {
                return release_date;
        }

        /**
        * Gets the name for this DbInfo.
        *
        * @return        name for this DbInfo.
        *
        * @since         DTS 2.4
        */
        public String toString()
        {
                return getName();
        }

        /**
        * Tests if two DbInfos are equal. Two DbInfos are equal if:
        * <br> they are of the same class
        * <br> their names are equal
        * <br> their versions are equal
        * <br> their dates are equal<P>
        *
        * @param         info               one DbInfo
        * @return        true if info equals to this DbInfo, false if info doesn't
        *                equal to this DbInfo.
        *
        * @since         DTS 2.4
        */
        public boolean equals(DbInfo info)
        {
                return (this.getClass().getName().equals(info.getClass().getName()) &&
                        this.getName().equals(info.getName()) &&
                        this.getVersion().equals(info.getVersion()) &&
                        this.getReleaseDate().toString().equals(info.getReleaseDate().toString()));
        }

        /**
        * Creates a DbInfo by passing an XML Document.
        *
        * @param       doc       the XML document of the DbInfo.
        * @return      the created DbInfo.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static DbInfo createDbInfo(Document doc) throws DTSException
        {
                Element elem = doc.getDocumentElement();
                cat.debug("DBINFO: " + elem.getNodeValue());
                return createDbInfo(elem);
        }
        /**
        * Creates a DbInfo by passing an XML Element. The Element contains
        * the information(s) about the DbInfo.
        *
        * @param       elem       the XML Element of the DbInfo.
        * @return      the created DbInfo.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static DbInfo createDbInfo(Element elem) throws DTSException
        {
                String tag_name = elem.getTagName().toString().toUpperCase();
                if (tag_name.equals("NULL"))
                        return null;
                if (tag_name.equals("DBINFO")) {
                        String s1 = elem.getChildNodes().item(0).getChildNodes().item(0).getNodeValue();
                        String s2 = elem.getChildNodes().item(1).getChildNodes().item(0).getNodeValue();
                        String s3 = elem.getChildNodes().item(2).getChildNodes().item(0).getNodeValue();

                        Date rdate = java.sql.Date.valueOf(s3);
                        DbInfo dbinfo = new DbInfo(s1, s2, rdate);
                        return dbinfo;
                } else {
                        throw new DTSException("Expected a DBINFO tag");
                }
        }

        /**
        * Creates DbInfos by passing an XML Document.
        *
        * @param       doc       the XML document of DbInfos.
        * @return      an array of the created DbInfos.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static DbInfo[] createDbInfos(Document doc) throws DTSException {
                Element elem = doc.getDocumentElement();
                cat.debug("DBINFOS: " + elem.getNodeValue());
                return createDbInfos(elem);
        }
        /**
        * Creates DbInfos by passing an XML Element. The Element contains
        * informations about DbInfos.
        *
        * @param       elem       the XML Element of DbInfos.
        * @return      an array of the created DbInfos.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static DbInfo[] createDbInfos(Element elem) throws DTSException
        {
                String tag_name = elem.getTagName().toString().toUpperCase();
                if(tag_name.equals("NULL")) {
                        return new DbInfo[0];
                } else if (tag_name.equals("DBINFOS")) {
                        int n = elem.getChildNodes().getLength();
                        DbInfo[] dbinfos = new DbInfo[n];
                        int i;
                        for (i=0; i<n; i++) {
	                        dbinfos[i] = createDbInfo((Element)elem.getChildNodes().item(i));
                        }
                        return dbinfos;
                } else {
                        throw new DTSException("Expected a DBINFOS tag");
                }
        }
}
