package org.nci.dts.db;

import java.sql.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import com.apelon.common.xml.*;
import org.nci.dts.EditActionConstants;
import org.w3c.dom.Node;
/**
 * <p>ConceptHistoryDb runs querys against Oracle on behalf of org.nci.dts.server.QueryServer,
 * occasionally gets the right answers and returns them as XML streams.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version 1.0
 */

public class ConceptHistoryDb {

        protected Connection conn;
        Statement stmt;

        /**
         * Constructs a ConceptHistoryDb.
         * @param conn
         * @throws SQLException
         */
        public ConceptHistoryDb(Connection conn) throws SQLException
        {
                this.conn = conn;
                stmt = conn.createStatement();
        }

        /**
         * Closes the connection.
         * @throws SQLException
         */
        public void close() throws SQLException
        {
                conn.close();
                if (stmt != null) stmt.close();
                stmt = null;
        }

        /**
         * Gets the date(s) in the concept_history table.
         * <p>This method is corresponding to the getHistoryDates method, the getHistoryStart
         * method, and the getHistoryEnd method in org.nci.dts.client.ConceptHistoryServer.</p>
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String executeGetDate(Element root) throws SQLException, XMLException
        {
                Element target = (Element) root.getFirstChild();
                String query = "";
                String value_type = target.getAttribute("value").toString().toUpperCase();
                String target_name = target.getTagName().toString().toUpperCase();

                if (target_name.equals("DATE")) {
                        boolean one = true;

                        if (value_type.equals("START")) {
                                query += " SELECT MIN(" + dateColumnName() + ") AS editdate ";
                        } else if (value_type.equals("END")) {
                                query += " SELECT MAX(" + dateColumnName() + ") AS editdate ";
                        } else if (value_type.equals("ALL")) {
                                query += " SELECT DISTINCT " + dateColumnName();
                                one = false;
                        }
                        query += " FROM " + historyTblName();

                        PreparedStatement stmt = conn.prepareStatement(query);
                        String res = executeDateQuery(stmt, one);
                        stmt.close();
                        return res;
                } else {
                        throw new XMLException("unrecognized target", target);
                }
        }

        /**
         * Gets the concept edit date(s).
         * <p>This method is corresponding to the getConceptCreationDate method and
         * the getConceptEditActionDates method in org.nci.dts.client.ConceptHistoryServer.</p>
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String executeGetConceptDate(Element root) throws SQLException, XMLException
        {
                Element target = (Element) root.getChildNodes().item(0);
                Element conceptCodeElem = (Element) root.getChildNodes().item(1);
                String query = "";

                String conceptCode = conceptCodeElem.getFirstChild().getNodeValue();
                String namespaceId = conceptCodeElem.getAttribute("namespaceid");

                String target_name = target.getTagName().toString().toUpperCase();
                PreparedStatement stmt;
                boolean one = true;
                if (target_name.equals("CREATEDATE")) {
                        query += " SELECT " + dateColumnName();
                        query += " FROM " + historyTblName();
                        query += " WHERE " + conceptColumnName() + " = ? ";
                        query += "   AND " + actionColumnName() + " = 'create' ";
                        query += "   AND " + namespaceColumnName() + " = ? ";

                        stmt = conn.prepareStatement(query);
                        stmt.setString(1, conceptCode);
                        stmt.setInt(2, Integer.parseInt(namespaceId));
                } else if (target_name.equals("EDITDATE")) {
                        String value = target.getFirstChild().getNodeValue();

                        query += " SELECT DISTINCT " + dateColumnName();
                        query += " FROM " + historyTblName();
                        query += " WHERE " + conceptColumnName() + " = ? ";
                        query += " AND " + actionColumnName() + " = ? ";
                        query += "   AND " + namespaceColumnName() + " = ? ";

                        stmt = conn.prepareStatement(query);
                        stmt.setString(1, conceptCode);
                        stmt.setString(2, value);
                        stmt.setInt(3, Integer.parseInt(namespaceId));

                        one = false;
                } else {
                        throw new XMLException("unrecognized target", target);
                }

                String res = executeDateQuery(stmt, one);
                stmt.close();
                return res;
        }

        /**
         * Gets the descendant/ancestor concept codes in the date range.
         * <p>This method is corresponding to the getDescendantCodes method and
         * the getAncestorCodes method in org.nci.dts.client.ConceptHistoryServer.</p>
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String executeGetCodesInDateRange(Element root) throws SQLException, XMLException
        {
                Element target = (Element) root.getChildNodes().item(0);
                Element conceptCodeElem = (Element) root.getChildNodes().item(1);
                Element atBaselineElem = (Element) root.getChildNodes().item(2);
                Element startDateElem = (Element) root.getChildNodes().item(3);
                Element endDateElem = (Element) root.getChildNodes().item(4);

                String target_name = target.getTagName().toString().toUpperCase();
                String conceptCode = conceptCodeElem.getFirstChild().getNodeValue();
                String namespaceId = conceptCodeElem.getAttribute("namespaceid");
                String atBaseline = atBaselineElem.getFirstChild().getNodeValue();
                String startDate_str = startDateElem.getFirstChild().getNodeValue();
                String endDate_str = endDateElem.getFirstChild().getNodeValue();

                java.sql.Date startDate = java.sql.Date.valueOf(startDate_str);
                java.sql.Date endDate = java.sql.Date.valueOf(endDate_str);

                String query = "";
                PreparedStatement stmt;
                if (target_name.equals("DESCENDANT")) {
                        query += " SELECT DISTINCT " + referenceColumnName();
                        query += " FROM " + historyTblName();
                        query += " WHERE " + conceptColumnName() + " = ? ";
                        query += " AND " + namespaceColumnName() + " = ? ";
                        query += " AND " + actionColumnName() + " in ('";
                        query += EditActionConstants.EDIT_ACTION_SPLIT_STR + "', '";
                        query += EditActionConstants.EDIT_ACTION_MERGE_STR + "') ";
                        query += " AND " + dateColumnName() + " >= ? ";
                        query += " AND " + dateColumnName() + " <= ? ";
                        if (atBaseline.equalsIgnoreCase("true")) {
                                query += " MINUS ( SELECT DISTINCT " + conceptColumnName();
                                query += "         FROM " + historyTblName();
                                query += "         WHERE " + actionColumnName() + " = '";
                                query += EditActionConstants.EDIT_ACTION_RETIRE_STR + "'";
                                query += "         AND " + dateColumnName() + " >= ? ";
                                query += "         AND " + dateColumnName() + " <= ? )";
                        }
                } else if (target_name.equals("ANCESTOR")) {
                        query += " SELECT DISTINCT " + conceptColumnName();
                        query += " FROM " + historyTblName();
                        query += " WHERE " + referenceColumnName() + " = ? ";
                        query += " AND " + namespaceColumnName() + " = ? ";
                        query += " AND " + actionColumnName() + " in ('";
                        query += EditActionConstants.EDIT_ACTION_SPLIT_STR + "', '";
                        query += EditActionConstants.EDIT_ACTION_MERGE_STR + "') ";
                        query += " AND " + dateColumnName() + " >= ? ";
                        query += " AND " + dateColumnName() + " <= ? ";
                        if (atBaseline.equalsIgnoreCase("true")) {
                                query += " MINUS ( SELECT DISTINCT " + conceptColumnName();
                                query += "         FROM " + historyTblName();
                                query += "         WHERE " + actionColumnName() + " = '";
                                query += EditActionConstants.EDIT_ACTION_RETIRE_STR + "'";
                                query += "         AND " + dateColumnName() + " <= ? )";
                        }
                } else {
                        throw new XMLException("unrecognized target", target);
                }

                stmt = conn.prepareStatement(query);
                stmt.setString(1, conceptCode);
                stmt.setInt(2, Integer.parseInt(namespaceId));
                stmt.setDate(3, startDate);
                stmt.setDate(4, endDate);
                if (atBaseline.equalsIgnoreCase("true")) {
                        stmt.setDate(5, startDate);
                        if (target_name.equals("DESCENDANT")) {
                                stmt.setDate(6, endDate);
                        }
                }
                String res = executeConceptQuery(stmt);
                stmt.close();
                return res;
        }

        /**
         * Gets the descendant/ancestor concept codes in the baseline date.
         * <p>This method is corresponding to the getCodeActionChildren method and
         * the getCodeActionParents method in org.nci.dts.client.ConceptHistoryServer.</p>
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String executeGetCodesInDate(Element root) throws SQLException, XMLException
        {
                Element target = (Element) root.getChildNodes().item(0);
                Element conceptCodeElem = (Element) root.getChildNodes().item(1);
                Element baselineDateElem = (Element) root.getChildNodes().item(2);

                String target_name = target.getTagName().toString().toUpperCase();

                String conceptCode = conceptCodeElem.getFirstChild().getNodeValue();
                String namespaceId = conceptCodeElem.getAttribute("namespaceid");
                String baselineDate = baselineDateElem.getFirstChild().getNodeValue();

                String query = "";
                PreparedStatement stmt;
                if (target_name.equals("DESCENDANT")) {
                        query += " SELECT DISTINCT " + referenceColumnName();
                        query += " FROM " + historyTblName();
                        query += " WHERE " + conceptColumnName() + " = ? ";
                        query += " AND " + namespaceColumnName() + " = ? ";
                        query += " AND " + actionColumnName() + " in ('";
                        query += EditActionConstants.EDIT_ACTION_SPLIT_STR + "', '";
                        query += EditActionConstants.EDIT_ACTION_MERGE_STR + "') ";
                        query += " AND " + dateColumnName() + " = ? ";
                        query += " MINUS ( SELECT DISTINCT " + conceptColumnName();
                        query += "         FROM " + historyTblName();
                        query += "         WHERE " + actionColumnName() + " = '";
                        query += EditActionConstants.EDIT_ACTION_RETIRE_STR + "'";
                        query += "         AND " + dateColumnName() + " = ? ";
                        query += "         AND " + conceptColumnName() + " = ? ";
                        query += "         AND " + namespaceColumnName() + " = ? )";

                } else if (target_name.equals("ANCESTOR")) {
                        query += " SELECT DISTINCT " + conceptColumnName();
                        query += " FROM " + historyTblName();
                        query += " WHERE " + referenceColumnName() + " = ? ";
                        query += " AND " + namespaceColumnName() + " = ? ";
                        query += " AND " + actionColumnName() + " in ('";
                        query += EditActionConstants.EDIT_ACTION_SPLIT_STR + "', '";
                        query += EditActionConstants.EDIT_ACTION_MERGE_STR + "') ";
                        query += " AND " + dateColumnName() + " = ? ";

                } else {
                        throw new XMLException("unrecognized target", target);
                }

                stmt = conn.prepareStatement(query);
                stmt.setString(1, conceptCode);
                stmt.setInt(2, Integer.parseInt(namespaceId));
                stmt.setDate(3, java.sql.Date.valueOf(baselineDate));
                if (target_name.equals("DESCENDANT")) {
                        stmt.setDate(4, java.sql.Date.valueOf(baselineDate));
                        stmt.setString(5, conceptCode);
                        stmt.setInt(6, Integer.parseInt(namespaceId));
                }
                String res = executeConceptQuery(stmt);
                stmt.close();
                return res;
        }

        /**
         * Gets the descendant/ancestor's concept codes and the corresponding baseline
         * dates in the date range.
         * <p>This method is corresponding to the getDescendantCodeActionDates method and
         * the getAncestorCodeActionDates method in org.nci.dts.client.ConceptHistoryServer.</p>
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String executeGetCodeActionDate(Element root) throws SQLException, XMLException
        {
                Element target = (Element) root.getChildNodes().item(0);
                Element conceptCodeElem = (Element) root.getChildNodes().item(1);
                Element startDateElem = (Element) root.getChildNodes().item(2);
                Element endDateElem = (Element) root.getChildNodes().item(3);

                String conceptCode = conceptCodeElem.getFirstChild().getNodeValue();
                String namespaceId = conceptCodeElem.getAttribute("namespaceid");
                String target_name = target.getTagName().toString().toUpperCase();
                String startDate = startDateElem.getFirstChild().getNodeValue();
                String endDate = endDateElem.getFirstChild().getNodeValue();

                String query = "";
                PreparedStatement stmt;
                if (target_name.equals("DESCENDANT")) {
                        query += " SELECT " + referenceColumnName() + ", " + dateColumnName();
                        query += " FROM " + historyTblName();
                        query += " WHERE " + conceptColumnName() + " = ? ";
                        query += " AND " + namespaceColumnName() + " = ? ";
                        query += " AND " + actionColumnName() + " in ('";
                        query += EditActionConstants.EDIT_ACTION_SPLIT_STR + "', '";
                        query += EditActionConstants.EDIT_ACTION_MERGE_STR + "') ";
                        query += " AND " + dateColumnName() + " >= ? ";
                        query += " AND " + dateColumnName() + " <= ? ";
                        query += " ORDER BY " + dateColumnName();
                } else if (target_name.equals("ANCESTOR")) {
                        query += " SELECT " + conceptColumnName() + ", " + dateColumnName();
                        query += " FROM " + historyTblName();
                        query += " WHERE " + referenceColumnName() + " = ? ";
                        query += " AND " + namespaceColumnName() + " = ? ";
                        query += " AND " + actionColumnName() + " in ('";
                        query += EditActionConstants.EDIT_ACTION_SPLIT_STR + "', '";
                        query += EditActionConstants.EDIT_ACTION_MERGE_STR + "') ";
                        query += " AND " + dateColumnName() + " >= ? ";
                        query += " AND " + dateColumnName() + " <= ? ";
                        query += " ORDER BY " + dateColumnName();
                } else {
                        throw new XMLException("unrecognized target", target);
                }

                stmt = conn.prepareStatement(query);
                stmt.setString(1, conceptCode);
                stmt.setInt(2, Integer.parseInt(namespaceId));
                stmt.setDate(3, java.sql.Date.valueOf(startDate));
                stmt.setDate(4, java.sql.Date.valueOf(endDate));
                String res = executeCodeActionDateQuery(stmt);
                stmt.close();
                return res;
        }

        /**
         * Gets the specified concept's edit actions and the corresponding edit dates.
         * <p>This method is corresponding to the getConceptEditAction method in
         * org.nci.dts.client.ConceptHistoryServer.</p>
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String executeGetEditAction(Element root) throws SQLException, XMLException
        {
                Element conceptCodeElem = (Element) root.getChildNodes().item(0);

                String conceptCode = conceptCodeElem.getFirstChild().getNodeValue();
                String namespaceId = conceptCodeElem.getAttribute("namespaceid");

                String query = "";
                PreparedStatement stmt;
                query += " SELECT DISTINCT " + dateColumnName() + ", " + actionColumnName();
                query += " FROM " + historyTblName();
                query += " WHERE " + conceptColumnName() + " = ? ";
                query += " AND " + namespaceColumnName() + " = ? ";
                query += " ORDER BY " + dateColumnName();

                stmt = conn.prepareStatement(query);
                stmt.setString(1, conceptCode);
                stmt.setInt(2, Integer.parseInt(namespaceId));

                String res = executeEditActionQuery(stmt);
                stmt.close();
                return res;
        }

        /**
         * Checks if the specified concept is retired or not.
         * <p>This method is corresponding to the isConceptRetired method in
         * org.nci.dts.client.ConceptHistoryServer.</p>
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String executeIsRetired(Element root) throws SQLException, XMLException
        {
                Element conceptCodeElem = (Element) root.getChildNodes().item(0);

                String conceptCode = conceptCodeElem.getFirstChild().getNodeValue();
                String namespaceId = conceptCodeElem.getAttribute("namespaceid");

                String query = "";
                PreparedStatement stmt;
                query += " SELECT count(*)";
                query += " FROM " + historyTblName();
                query += " WHERE " + conceptColumnName() + " = ? ";
                query += " AND " + namespaceColumnName() + " = ? ";
                query += " AND " + actionColumnName() + " = '" + EditActionConstants.EDIT_ACTION_RETIRE_STR + "'";

                stmt = conn.prepareStatement(query);
                stmt.setString(1, conceptCode);
                stmt.setInt(2, Integer.parseInt(namespaceId));
                String res = executeBooleanQuery(stmt);
                stmt.close();
                return res;
        }

        /**
         * Checks if the specified concept is edited or not in the date range. There is a
         * value to specify the checking type: either checking all edit actions or only
         * checking split/merge actions.
         * <p>This method is corresponding to the isConceptEdited method in
         * org.nci.dts.client.ConceptHistoryServer.</p>
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String executeIsEdited(Element root) throws SQLException, XMLException
        {
                Element conceptCodeElem = (Element) root.getChildNodes().item(0);
                Element mrg_spltElem = (Element) root.getChildNodes().item(1);
                Element startDateElem = (Element) root.getChildNodes().item(2);
                Element endDateElem = (Element) root.getChildNodes().item(3);

                String conceptCode = conceptCodeElem.getFirstChild().getNodeValue();
                String namespaceId = conceptCodeElem.getAttribute("namespaceid");
                String mrg_splt = mrg_spltElem.getFirstChild().getNodeValue();
                String startDate = startDateElem.getFirstChild().getNodeValue();
                String endDate = endDateElem.getFirstChild().getNodeValue();

                String query = "";
                PreparedStatement stmt;
                query += " SELECT count(*)";
                query += " FROM " + historyTblName();
                query += " WHERE " + conceptColumnName() + " = ? ";
                query += " AND " + namespaceColumnName() + " = ? ";
                query += " AND " + dateColumnName() + " >= ? ";
                query += " AND " + dateColumnName() + " <= ? ";
                if (mrg_splt.equalsIgnoreCase("true")) {
                        query += " AND " + actionColumnName() + " in ('";
                        query += EditActionConstants.EDIT_ACTION_SPLIT_STR + "', '";
                        query += EditActionConstants.EDIT_ACTION_MERGE_STR + "') ";
                }

                stmt = conn.prepareStatement(query);
                stmt.setString(1, conceptCode);
                stmt.setInt(2, Integer.parseInt(namespaceId));
                stmt.setDate(3, java.sql.Date.valueOf(startDate));
                stmt.setDate(4, java.sql.Date.valueOf(endDate));
                String res = executeBooleanQuery(stmt);
                stmt.close();
                return res;
        }

        /**
         * Gets the specified concept's edit actions in the baseline date.
         * <p>This method is corresponding to the getNumberEditActions method which has two parameters
         * and the getConceptEditAction method in org.nci.dts.client.ConceptHistoryServer.</p>
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String executeGetAction(Element root) throws SQLException, XMLException
        {
                String search_type = ((String) root.getAttribute("type")).toUpperCase();
                Element conceptCodeElem = (Element) root.getChildNodes().item(0);
                Element baselineDateElem = (Element) root.getChildNodes().item(1);

                String conceptCode = conceptCodeElem.getFirstChild().getNodeValue();
                String namespaceId = conceptCodeElem.getAttribute("namespaceid");
                String baselineDate = baselineDateElem.getFirstChild().getNodeValue();

                String query = "";
                PreparedStatement stmt;
                if (search_type.equals("COUNT")) {
                        query += " SELECT count(*) ";
                        query += " FROM " + historyTblName();
                        query += " WHERE " + conceptColumnName() + " = ? ";
                        query += " AND " + namespaceColumnName() + " = ? ";
                        query += " AND " + dateColumnName() + " = ? ";
                        stmt = conn.prepareStatement(query);
                        stmt.setString(1, conceptCode);
                        stmt.setInt(2, Integer.parseInt(namespaceId));
                        stmt.setDate(3, java.sql.Date.valueOf(baselineDate));
                        String res = executeNumberActionQuery(stmt);
                        stmt.close();
                        return res;
                } else if (search_type.equals("DETAIL")) {
                        query += " SELECT DISTINCT " + actionColumnName();
                        query += " FROM " + historyTblName();
                        query += " WHERE " + conceptColumnName() + " = ? ";
                        query += " AND " + namespaceColumnName() + " = ? ";
                        query += " AND " + dateColumnName() + " = ? ";
                        stmt = conn.prepareStatement(query);
                        stmt.setString(1, conceptCode);
                        stmt.setInt(2, Integer.parseInt(namespaceId));
                        stmt.setDate(3, java.sql.Date.valueOf(baselineDate));
                        String res = executeActionsQuery(stmt);
                        stmt.close();
                        return res;
                } else {
                        throw new XMLException("unrecognized search type (either 'count' or 'detail').");
                }
        }

        /**
         * Gets the specified concept's edit actions in the date range.
         * <p>This method is corresponding to the other getNumberEditActions method which
         * has three parameters in org.nci.dts.client.ConceptHistoryServer.</p>
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String executeGetActionInDateRange(Element root) throws SQLException, XMLException
        {
                Element conceptCodeElem = (Element) root.getChildNodes().item(0);
                Element startDateElem = (Element) root.getChildNodes().item(1);
                Element endDateElem = (Element) root.getChildNodes().item(2);

                String conceptCode = conceptCodeElem.getFirstChild().getNodeValue();
                String namespaceId = conceptCodeElem.getAttribute("namespaceid");
                String startDate = startDateElem.getFirstChild().getNodeValue();
                String endDate = endDateElem.getFirstChild().getNodeValue();

                String query = "";
                PreparedStatement stmt;

                query += " SELECT count(*) ";
                query += " FROM " + historyTblName();
                query += " WHERE " + conceptColumnName() + " = ? ";
                query += " AND " + namespaceColumnName() + " = ? ";
                query += " AND " + dateColumnName() + " >= ? ";
                query += " AND " + dateColumnName() + " <= ? ";
                stmt = conn.prepareStatement(query);
                stmt.setString(1, conceptCode);
                stmt.setInt(2, Integer.parseInt(namespaceId));
                stmt.setDate(3, java.sql.Date.valueOf(startDate));
                stmt.setDate(4, java.sql.Date.valueOf(endDate));
                String res = executeNumberActionQuery(stmt);
                stmt.close();
                return res;
        }

        /**
         * Checks if the connection is closed.
         * @return  a boolean.    True - the connection is closed.
         *                        False - the connection isn't closed.
         * @throws SQLException
         */
        public boolean isClosed() throws SQLException { return conn.isClosed(); }

        /**
         * Executes the ping. According to the client side query, this method will do
         * either ping DTS server or ping database.
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         */
        public String executePing(Element root) throws SQLException {
                String bool_text = root.getFirstChild().getNodeValue();
                return executePing(Boolean.valueOf(bool_text).booleanValue());
        }

        /**
         * Executes the ping. According to the client side query, this method will do
         * either ping DTS server or ping database.
         * @param   db    True - ping database.
         *                False - ping DTS server.
         * @return  the XML formatted result.
         * @throws SQLException
         */
        protected String executePing(boolean db)
            throws SQLException
        {
                StringBuffer result = new StringBuffer();
                ResultSet res = null;
                result.append(XML.createProlog("string", org.nci.dts.dtd.DTD.RESULT, false));
                result.append("<string>");
                if (db)
                {
                        Statement stmt = conn.createStatement();
                        res = stmt.executeQuery("SELECT SYSDATE FROM DUAL");
                        while (res.next())
                        {
                                String time = res.getString(1);
                                result.append(time);
                        }
                        res.close();
                        stmt.close();
                }
                else
                {
                        result.append((new java.util.Date()).toString());
                }
                result.append("</string>");
                return result.toString();
        }

        protected String executeDateQuery(PreparedStatement stmt, boolean one)
            throws SQLException
        {
                return executeQuery(stmt, "editDate", one, true);
        }

        protected String executeConceptQuery(PreparedStatement stmt)
            throws SQLException
        {
                return executeQuery(stmt, "conceptCode", false, false);
        }

        protected String executeCodeActionDateQuery(PreparedStatement stmt)
            throws SQLException
        {
                return executeQuery(stmt, "codeActionDate", "conceptCode", false, "editDate", true, false);
        }

        protected String executeEditActionQuery(PreparedStatement stmt)
            throws SQLException
        {
                return executeQuery(stmt, "editAction", "editDate", true, "action", false, false);
        }

        protected String executeNumberActionQuery(PreparedStatement stmt)
            throws SQLException
        {
                return executeQuery(stmt, "string", true, false);
        }

        protected String executeActionsQuery(PreparedStatement stmt)
            throws SQLException
        {
                return executeQuery(stmt, "action", false, false);
        }

        protected String executeBooleanQuery(PreparedStatement stmt)
            throws SQLException
        {
                StringBuffer result = new StringBuffer();
                ResultSet res = stmt.executeQuery();

                if(!res.next()) {
                        res.close();
                        stmt.close();
                        return nullDocument();
                } else {
                        result.append(XML.createProlog("boolean", org.nci.dts.dtd.DTD.RESULT, false));
                        XML.asStartTag(result, "boolean");
                        boolean result_b = false;
                        int count = res.getInt(1);
                        if (count > 0 ) result_b = true;
                        XML.asEmptyTag(result, "" + result_b);
                        XML.asEndTag(result, "boolean");
                        res.close();
                        stmt.close();
                        return result.toString();
                }
        }

        private String executeQuery(PreparedStatement stmt, String tagName, boolean one, boolean isDate)
            throws SQLException
        {
                StringBuffer result = new StringBuffer(10000);
                String tagName1 = tagName + "s";

                if (one) {
                        result.append(XML.createProlog(tagName, org.nci.dts.dtd.DTD.RESULT, false));
                } else {
                        result.append(XML.createProlog(tagName1, org.nci.dts.dtd.DTD.RESULT, false));
                }
                if (!one) {
                        XML.asStartTag(result, tagName1);
                }

                ResultSet res = stmt.executeQuery();
                String value = "";
                if(!res.next()) {
                        res.close();
                        stmt.close();
                        return nullDocument();
                } else {
                        do {
                                if (isDate) {
                                        Date dt = res.getDate(1);
                                        value = dt.toString();
                                } else {
                                        value = res.getString(1);
                                }
                                appendElem(result, tagName, value);
                                if (one) break;
                        }while (res.next());
                }
                if (!one) {
                        XML.asEndTag(result, tagName1);
                }
                res.close();
                stmt.close();
                return result.toString();
        }

        private String executeQuery(PreparedStatement stmt, String tagName,
                                    String subTagName1, boolean subTagName1IsDate,
                                    String subTagName2, boolean subTagName2IsDate, boolean one)
            throws SQLException
        {
                StringBuffer result = new StringBuffer(10000);
                String tagName1 = tagName + "s";

                if (one) {
                        result.append(XML.createProlog(tagName, org.nci.dts.dtd.DTD.RESULT, false));
                } else {
                        result.append(XML.createProlog(tagName1, org.nci.dts.dtd.DTD.RESULT, false));
                }
                if (!one) {
                        XML.asStartTag(result, tagName1);
                }

                ResultSet res = stmt.executeQuery();
                String value1 = "";
                String value2 = "";
                if(!res.next()) {
                        res.close();
                        stmt.close();
                        return nullDocument();
                } else {
                        do {
                                if (subTagName1IsDate) {
                                        Date dt1 = res.getDate(1);
                                        value1 = dt1.toString();
                                } else {
                                        value1 = res.getString(1);
                                }
                                if (subTagName2IsDate) {
                                        Date dt2 = res.getDate(2);
                                        value2 = dt2.toString();
                                } else {
                                        value2 = res.getString(2);
                                }
                                XML.asStartTag(result, tagName);
                                appendElem(result, subTagName1, value1);
                                appendElem(result, subTagName2, value2);
                                XML.asEndTag(result, tagName);
                                if (one) break;
                        }while (res.next());
                }
                if (!one) {
                        XML.asEndTag(result, tagName1);
                }
                res.close();
                stmt.close();
                return result.toString();
        }

        private String nullDocument()
        {
                StringBuffer result = new StringBuffer();
                String nullElementName = "null";
                result.append(XML.createProlog(nullElementName, org.nci.dts.dtd.DTD.RESULT, false));
                result.append( XML.asEmptyTag(nullElementName));
                return result.toString();
        }

        protected void appendElem(StringBuffer docString, String tag, String value) {
                XML.asTagValue(docString, tag, value);
        }

        private String historyTblName () { return "concept_history"; }
        private String idColumnName () { return "id"; }
        private String conceptColumnName () { return "concept"; }
        private String namespaceColumnName () { return "namespaceid"; }
        private String actionColumnName () { return "editaction"; }
        private String dateColumnName () { return "editdate"; }
        private String referenceColumnName () { return "reference"; }

}