package org.nci.dts.db;

import java.sql.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import com.apelon.common.xml.*;
import com.apelon.common.sql.SQL;
import org.nci.dts.dtd.DTD;
 /**
 * <p>DbInfoDb runs querys against Oracle on behalf of org.nci.dts.server.QueryServer,
 * occasionally gets the right answers and returns them as XML streams.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version 1.0
 */

public class DbInfoDb
{
        private boolean DbInfoTablePresent = false;
        private String DBINFO_TABLE = "DB_INFO";
        protected Statement stmt;
        protected Connection conn;

        /**
         * Constructs a DbInfoDb.
         * @param conn
         * @throws SQLException
         */
        public DbInfoDb (Connection conn) throws SQLException
        {
                this.conn = conn;
                stmt = conn.createStatement();
                DbInfoTablePresent = SQL.checkTableExists(conn,DBINFO_TABLE);
        }

        /**
         * Closes the connection.
         * @throws SQLException
         */
        public void close() throws SQLException
        {
                conn.close();
                if (stmt != null) stmt.close();
                stmt = null;
        }

        /**
         * Checks if the connection is closed.
         * @return  a boolean.    True - the connection is closed.
         *                        False - the connection isn't closed.
         * @throws SQLException
         */
        public boolean isClosed() throws SQLException { return conn.isClosed(); }

        /**
         * Gets the database information including database name, version, and release date.
         * <p>This method is corresponding to the getDbInfo method and the getDbInfos method
         * in org.nci.dts.client.DbInfoServer.</p>
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String executeGetDbInfo(Element root) throws SQLException, XMLException
        {
                Element target = ((Element) root.getFirstChild());
                String value = null;
                if (target.hasChildNodes()) {
                        value = target.getFirstChild().getNodeValue();
                }
                String value_type = target.getAttribute("value").toString().toUpperCase();
                String target_name = target.getTagName().toString().toUpperCase();
                String query = "";
                if( !DbInfoTablePresent ) {
                        throw new XMLException("DB_INFO table doesn't exist!", target);
                } else if (target_name.equals("DBINFO")) {
                        boolean one = true;
                        query += " SELECT " + NameCol() + "," + VersionCol() + "," + DateCol();
                        query += " FROM " + dbinfoTblName();
                        if (value_type.equals("NAME")) {
                                query += " WHERE " + NameCol() + " = '" + value + "'";
                        } else if (value_type.equals("ALL")) {
                                one = false;
                        } else {
                                throw new XMLException("unrecognized value", target);
                        }
                        return executeDbinfoQuery(query, one);
                }
                throw new XMLException("unrecognized target", target);
        }

        protected String executeDbinfoQuery(String sql, boolean one) throws SQLException
        {
                String dbinfosElem = "dbInfos";
                String dbinfoElem = "dbInfo";
                StringBuffer result = new StringBuffer();
                if(!one) {
                        result.append(XML.createProlog(dbinfosElem, org.nci.dts.dtd.DTD.RESULT, false));
                        XML.asStartTag(result, dbinfosElem);
                } else {
                        result.append(XML.createProlog(dbinfoElem, org.nci.dts.dtd.DTD.RESULT, false));
                }
                Statement stmt = conn.createStatement();
                ResultSet res = stmt.executeQuery(sql);
                if(!res.next()) {
                        res.close();
                        stmt.close();
                        return nullDocument();
                } else {
                        do {
                                String dbname = res.getString(1);
                                String version = res.getString(2);
                                Date date = res.getDate(3);
                                XML.asStartTag(result, dbinfoElem);
                                appendNameElem(result, dbname);
                                appendVersionElem(result, version);
                                appendDateElem(result, date.toString());
                                XML.asEndTag(result, dbinfoElem);
                                if (one) { break; }
                        } while (res.next());
                }
                if (!one) { XML.asEndTag(result, dbinfosElem); }
                res.close();
                stmt.close();
                return result.toString();
        }

        /**
         * Executes the ping. According to the client side query, this method will do
         * either ping DTS server or ping database.
         * @param   root    the XML Element of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         */
        public String executePing(Element root) throws SQLException
        {
                String bool_text = root.getFirstChild().getNodeValue();
                return executePing(Boolean.valueOf(bool_text).booleanValue());
        }

        /**
         * Executes the ping. According to the client side query, this method will do
         * either ping DTS server or ping database.
         * @param   db    True - ping database.
         *                False - ping DTS server.
         * @return  the XML formatted result.
         * @throws SQLException
         */
        protected String executePing(boolean db)
            throws SQLException
        {
                StringBuffer result = new StringBuffer();
                ResultSet res = null;
                result.append(XML.createProlog("string", org.nci.dts.dtd.DTD.RESULT, false));
                result.append("<string>");
                if (db) {
                        Statement stmt = conn.createStatement();
                        res = stmt.executeQuery("SELECT SYSDATE FROM DUAL");
                        while (res.next()) {
                                String time = res.getString(1);
                                result.append(time);
                        }
                        res.close();
                        stmt.close();
                } else {
                        result.append((new java.util.Date()).toString());
                }
                result.append("</string>");
                return result.toString();
        }

        protected String nullDocument()
        {
                StringBuffer result = new StringBuffer();
                String nullElementName = "null";
                result.append(XML.createProlog(nullElementName, org.nci.dts.dtd.DTD.RESULT, false));
                result.append( XML.asEmptyTag(nullElementName));
                return result.toString();
        }

        protected String NameCol() {
                return "db_name";
        }

        protected String VersionCol() {
                return "version";
        }

        protected String DateCol() {
                return "release_date";
        }

        protected String dbinfoTblName() {
                return DBINFO_TABLE;
        }

        protected void appendNameElem(StringBuffer docString, String value) {
                XML.asTagValue(docString, "name", value);
        }

        protected void appendVersionElem(StringBuffer docString, String value) {
                XML.asTagValue(docString, "version", value);
        }

        protected void appendDateElem(StringBuffer docString, String value) {
                XML.asTagValue(docString, "date", value);
        }
}
